@@ -1,7 +1,7 @@
 /*
  * mdadm - manage Linux "md" devices aka RAID arrays.
  *
- * Copyright (C) 2001-2004 Neil Brown <neilb@cse.unsw.edu.au>
+ * Copyright (C) 2001-2006 Neil Brown <neilb@suse.de>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -121,7 +121,7 @@ int Grow_Add_device(char *devname, int fd, char *newdev)
 	info.disk.minor = minor(stb.st_rdev);
 	info.disk.raid_disk = d;
 	info.disk.state = (1 << MD_DISK_SYNC) | (1 << MD_DISK_ACTIVE);
-	st->ss->update_super(&info, super, "grow", newdev, 0);
+	st->ss->update_super(&info, super, "grow", newdev, 0, 0, NULL);
 
 	if (st->ss->store_super(st, nfd, super)) {
 		fprintf(stderr, Name ": Cannot store new superblock on %s\n", newdev);
@@ -179,7 +179,7 @@ int Grow_Add_device(char *devname, int fd, char *newdev)
 		info.disk.minor = minor(stb.st_rdev);
 		info.disk.raid_disk = nd;
 		info.disk.state = (1 << MD_DISK_SYNC) | (1 << MD_DISK_ACTIVE);
-		st->ss->update_super(&info, super, "grow", dv, 0);
+		st->ss->update_super(&info, super, "grow", dv, 0, 0, NULL);
 		
 		if (st->ss->store_super(st, fd2, super)) {
 			fprintf(stderr, Name ": Cannot store new superblock on %s\n", dv);
@@ -272,7 +272,7 @@ int Grow_addbitmap(char *devname, int fd, char *file, int chunk, int delay, int
 	}
 
 	if (array.level == 10) {
-		int ncopies = (array.layout&255)*(array.layout>>8);
+		int ncopies = (array.layout&255)*((array.layout>>8)&255);
 		bitmapsize = bitmapsize * array.raid_disks / ncopies;
 	}
 
@@ -329,8 +329,6 @@ int Grow_addbitmap(char *devname, int fd, char *file, int chunk, int delay, int
 		int d;
 		int max_devs = st->max_devs;
 		void *super = NULL;
-		if (chunk == UnSet)
-			chunk = DEFAULT_BITMAP_CHUNK;
 
 		/* try to load a superblock */
 		for (d=0; d<max_devs; d++) {
@@ -659,7 +657,7 @@ int Grow_reshape(char *devname, int fd, int quiet, char *backup_file,
 				offsets[sd->role] = sd->offset;
 				if (fdlist[sd->role] < 0) {
 					fprintf(stderr, Name ": %s: cannot open component %s\n",
-						devname, dn);
+						devname, dn?dn:"-unknown-");
 					goto abort;
 				}
 			} else {
@@ -669,7 +667,7 @@ int Grow_reshape(char *devname, int fd, int quiet, char *backup_file,
 				offsets[d] = sd->offset;
 				if (fdlist[d]<0) {
 					fprintf(stderr, Name ": %s: cannot open component %s\n",
-						devname, dn);
+						devname, dn?dn:"-unknown");
 					goto abort;
 				}
 				d++;
@@ -803,7 +801,10 @@ int Grow_reshape(char *devname, int fd, int quiet, char *backup_file,
 		memset(&bsb, 0, sizeof(bsb));
 		for (i=odisks; i<d ; i++) {
 			lseek64(fdlist[i], (offsets[i]+last_block)<<9, 0);
-			write(fdlist[i], &bsb, sizeof(bsb));
+			if (write(fdlist[i], &bsb, sizeof(bsb)) < 0) {
+				fprintf(stderr, Name ": %s: failed to invalidate metadata for raid disk %d\n",
+					devname, i);
+			}
 		}
 
 		/* unsuspend. */
@@ -950,7 +951,7 @@ int Grow_restart(struct supertype *st, struct mdinfo *info, int *fdlist, int cnt
 				continue;
 			st->ss->getinfo_super(&dinfo, super);
 			dinfo.reshape_progress = __le64_to_cpu(bsb.length);
-			st->ss->update_super(&dinfo, super, "_reshape_progress",NULL,0);
+			st->ss->update_super(&dinfo, super, "_reshape_progress",NULL,0, 0, NULL);
 			st->ss->store_super(st, fdlist[j], super);
 			free(super);
 		}
